use tokio_postgres::Client as PgClient;
use std::sync::Arc;
use crate::models::rent::{BindingRent, Rent};
use crate::storages::traits::RentRepository;
use async_trait::async_trait;

pub struct PostgresRentRepository {
    pub connection: Arc<PgClient>
}

#[async_trait]
impl RentRepository for PostgresRentRepository {
    async fn create(&self, rent: BindingRent) -> Result<Rent, String> {
        let query_result = self.connection.query(
            "SELECT * FROM Rent WHERE car_id=$1 AND time_amount IS NOT NULL",
            &[&rent.car_id]
        ).await;

        if let Ok(rows) = query_result {
            if rows.len() > 0 {
                return Err("The car is already occupied".to_owned());
            }
        } else {
            return Err("Something gone wrong during checking for existing rents".to_owned());
        }

        let result = self.connection.query(
            "INSERT INTO Rent(time_amount, client_id, car_id) \
            VALUES ($1, $2, $3) RETURNING *",
            &[&rent.time_amount, &rent.client_id, &rent.car_id]
        ).await;

        if let Ok(rows) = result {
            let row = rows.get(0).unwrap();
            Ok(Rent {
                id: row.get("id"),
                start_time: row.get("start_time"),
                time_amount: row.get("time_amount"),
                client_id: row.get("client_id"),
                car_id: row.get("car_id")
            })
        } else {
            Err("Something gone wrong during creation of Rent".to_owned())
        }
    }

    async fn read(&self, id: u32) -> Result<Rent, String> {
        let result = self.connection.query(
            "SELECT * FROM Rent WHERE id = $1", &[&id]
        ).await;

        if let Ok(rows) = result {
            let row = rows.get(0).ok_or("Rent not found".to_owned())?;
            Ok(Rent {
                id: row.get("id"),
                start_time: row.get("start_time"),
                time_amount: row.get("time_amount"),
                client_id: row.get("client_id"),
                car_id: row.get("car_id")
            })
        } else {
            Err("Something gone wrong during reading of Rent".to_owned())
        }
    }

    async fn read_all(&self) -> Result<Vec<Rent>, String> {
        let result = self.connection.query(
            "SELECT * FROM Rent", &[]
        ).await;

        if let Ok(rows) = result {
            Ok(
                rows.into_iter().map(|r| Rent {
                    id: r.get("id"),
                    start_time: r.get("start_time"),
                    time_amount: r.get("time_amount"),
                    client_id: r.get("client_id"),
                    car_id: r.get("car_id")
                }).collect()
            )
        } else {
            Err("Something gone wrong during reading CarStations".to_owned())
        }
    }

    async fn update(&self, id: u32, rent: BindingRent) -> Result<Rent, String> {
        let result = self.connection.query(
            "UPDATE Rent SET time_amount = $2, client_id = $3, car_id = $4 WHERE id = $5 RETURNING *",
            &[&rent.time_amount, &rent.client_id, &rent.car_id, &id]
        ).await;

        if let Ok(rows) = result {
            let row = rows.get(0).unwrap();
            Ok(Rent {
                id: row.get("id"),
                start_time: row.get("start_time"),
                time_amount: row.get("time_amount"),
                client_id: row.get("client_id"),
                car_id: row.get("car_id")
            })
        } else {
            Err("Something gone wrong during updating of Rent".to_owned())
        }
    }

    async fn delete(&self, id: u32) -> Result<(), String> {
        let result = self.connection.execute(
            "DELETE FROM Rent WHERE id = $1", &[&id]
        ).await;

        if let Ok(_) = result {
            Ok(())
        } else {
            Err("Something gone wrong during deletion of Rent".to_owned())
        }
    }
}